@@ -49,13 +49,19 @@ class XML(object):
             return dict_var
         else:
             parsed = list()
+            i = 0
 
-            for item in root.findall(xmlns_rs):
+            mylist = root.findall(xmlns_rs)
+            for item in mylist:
                 parsed.append(self.__parse_element(item))
-                self.__parse_df(parsed)
+
+                if self.__parse_df({'xml_data': parsed, 'counter': i - len(parsed) + 1}):
+                    parsed.clear()
+
+                i += 1
 
             if self.read_handlers:
-                self.__parse_df(parsed)
+                self.__parse_df({'xml_data': parsed, 'counter': i - len(parsed) + 1}, ignore=True)
             else:
                 df = DataFrame(parsed)
                 return df.applymap(lambda x: x.strip() if isinstance(x, str) else x)
@@ -85,24 +91,33 @@ class XML(object):
             )
             xmlfile.write('\n</records>')
 
-    def read_event(self, buffer=1000):
+    def read_event(self, buffer=1000, **kwargs):
         def registerhandler(handler):
-            self.read_handlers.append([handler, buffer])
+            self.read_handlers.append([handler, buffer, kwargs])
             return handler
 
         return registerhandler
 
-    def __parse_df(self, data):
+    def __parse_df(self, data, ignore=False):
+        return_val = False
+
         if self.read_handlers:
-            df = DataFrame()
+            data['df'] = DataFrame()
 
             for read_handler in self.read_handlers:
-                if len(data) >= read_handler[1]:
-                    if df.empty:
-                        df = DataFrame(data)
-                        df = df.applymap(lambda x: x.strip() if isinstance(x, str) else x)
+                if ignore or len(data['xml_data']) >= read_handler[1]:
+                    return_val = True
 
-                    read_handler[0](df)
+                    if data['df'].empty:
+                        data['df'] = DataFrame(data['xml_data'])
+                        data['df'] = data['df'].applymap(lambda x: x.strip() if isinstance(x, str) else x)
+
+                    if read_handler[2]:
+                        read_handler[0](data, **read_handler[2])
+                    else:
+                        read_handler[0](data)
+
+        return return_val
 
     def __parse_element(self, element, parsed=None):
         if not parsed:
